@@ -1,7 +1,5 @@
 /* flock.cc.  NT specific implementation of advisory file locking.
 
-   Copyright 2003, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Red Hat, Inc.
-
    This file is part of Cygwin.
 
    This software is a copyrighted work licensed under the terms of the
@@ -803,7 +801,7 @@ lockf_t::create_lock_obj ()
 
       pinfo p (myself->ppid);
       if (!p)	/* No access or not a Cygwin parent. */
-      	return;
+	return;
 
       parent_proc = OpenProcess (PROCESS_DUP_HANDLE
 				 | PROCESS_CREATE_THREAD
@@ -1027,7 +1025,7 @@ fhandler_base::lock (int a_op, struct flock *fl)
 
     case SEEK_END:
       if (get_device () != FH_FS)
-      	start = 0;
+	start = 0;
       else
 	{
 	  NTSTATUS status;
@@ -1300,7 +1298,7 @@ lf_setlock (lockf_t *lock, inode_t *node, lockf_t **clean, HANDLE fhdl)
 	timeout = 100L;
 
       DWORD WAIT_SIGNAL_ARRIVED = WAIT_OBJECT_0 + wait_count;
-      set_signal_arrived here (w4[wait_count++]);
+      wait_signal_arrived here (w4[wait_count++]);
 
       DWORD WAIT_THREAD_CANCELED = WAIT_TIMEOUT + 1;
       HANDLE cancel_event = pthread::get_cancel_event ();
@@ -1738,7 +1736,7 @@ lf_split (lockf_t *lock1, lockf_t *lock2, lockf_t **split)
   splitlock = *split;
   assert (splitlock != NULL);
   *split = splitlock->lf_next;
-  memcpy (splitlock, lock1, sizeof *splitlock);
+  memcpy ((void *) splitlock, lock1, sizeof *splitlock);
   /* We have to unset the obj HANDLE here which has been copied by the
      above memcpy, so that the calling function recognizes the new object.
      See post-lf_split handling in lf_setlock and lf_clearlock. */
@@ -2008,18 +2006,16 @@ fhandler_disk_file::mand_lock (int a_op, struct flock *fl)
 	      thr->detach ();
 	      break;
 	    default:
-	      /* Signal arrived. */
-	      /* Starting with Vista, CancelSynchronousIo works, and we wait
-		 for the thread to exit.  lp.status will be either
-		 STATUS_SUCCESS, or STATUS_CANCELLED.  We only call
-		 NtUnlockFile in the first case.
-		 Prior to Vista, CancelSynchronousIo doesn't exist, so we
-		 terminated the thread and always call NtUnlockFile since
-		 lp.status was 0 to begin with. */
+	      /* Signal arrived.
+		 If CancelSynchronousIo works we wait for the thread to exit.
+		 lp.status will be either STATUS_SUCCESS, or STATUS_CANCELLED.
+		 We only call NtUnlockFile in the first case.
+		 If CancelSynchronousIo fails we terminated the thread and
+		 call NtUnlockFile since lp.status was 0 to begin with. */
 	      if (CancelSynchronousIo (thr->thread_handle ()))
 		thr->detach ();
 	      else
-	      	thr->terminate_thread ();
+		thr->terminate_thread ();
 	      if (NT_SUCCESS (lp.status))
 		NtUnlockFile (get_handle (), &io, &offset, &length, 0);
 	      /* Per SUSv4: If a signal is received while fcntl is waiting,
