@@ -21,15 +21,6 @@ details. */
 #include "cpuid.h"
 #include "clock.h"
 
-static long
-get_open_max (int in)
-{
-  long max = getdtablesize ();
-  if (max < OPEN_MAX)
-    max = OPEN_MAX;
-  return max;
-}
-
 static long
 get_page_size (int in)
 {
@@ -341,9 +332,11 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL1_ICACHE_ASSOC:
 	  if (cur_level == 1 && cache_type == 2)
 	    return assoc;
+	  break;
 	case _SC_LEVEL1_ICACHE_LINESIZE:
 	  if (cur_level == 1 && cache_type == 2)
 	    return linesize;
+	  break;
 	case _SC_LEVEL1_DCACHE_SIZE:
 	  if (cur_level == 1 && cache_type == 1)
 	    ret += assoc * part * linesize * sets;
@@ -351,9 +344,11 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL1_DCACHE_ASSOC:
 	  if (cur_level == 1 && cache_type == 1)
 	    return assoc;
+	  break;
 	case _SC_LEVEL1_DCACHE_LINESIZE:
 	  if (cur_level == 1 && cache_type == 1)
 	    return linesize;
+	  break;
 	case _SC_LEVEL2_CACHE_SIZE:
 	  if (cur_level == 2)
 	    ret += assoc * part * linesize * sets;
@@ -361,9 +356,11 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL2_CACHE_ASSOC:
 	  if (cur_level == 2)
 	    return assoc;
+	  break;
 	case _SC_LEVEL2_CACHE_LINESIZE:
 	  if (cur_level == 2)
 	    return linesize;
+	  break;
 	case _SC_LEVEL3_CACHE_SIZE:
 	  if (cur_level == 3)
 	    ret += assoc * part * linesize * sets;
@@ -371,9 +368,11 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL3_CACHE_ASSOC:
 	  if (cur_level == 3)
 	    return assoc;
+	  break;
 	case _SC_LEVEL3_CACHE_LINESIZE:
 	  if (cur_level == 3)
 	    return linesize;
+	  break;
 	}
     }
   return ret;
@@ -486,7 +485,8 @@ get_cpu_cache (int in)
   vendor_id[3] = 0;
   if (!strcmp ((char*) vendor_id, "GenuineIntel"))
     return get_cpu_cache_intel (in, maxf & 0xffff);
-  else if (!strcmp ((char*)vendor_id, "AuthenticAMD"))
+  else if (!strcmp ((char*)vendor_id, "AuthenticAMD")
+           || !strcmp((char*)vendor_id, "HygonGenuine"))
     {
       uint32_t maxe = 0, unused;
       cpuid (&maxe, &unused, &unused, &unused, 0x80000000);
@@ -511,7 +511,7 @@ static struct
   {cons, {c:CHILD_MAX}},		/*   1, _SC_CHILD_MAX */
   {cons, {c:CLOCKS_PER_SEC}},		/*   2, _SC_CLK_TCK */
   {cons, {c:NGROUPS_MAX}},		/*   3, _SC_NGROUPS_MAX */
-  {func, {f:get_open_max}},		/*   4, _SC_OPEN_MAX */
+  {cons, {c:OPEN_MAX}},		/*   4, _SC_OPEN_MAX */
   {cons, {c:_POSIX_JOB_CONTROL}},	/*   5, _SC_JOB_CONTROL */
   {cons, {c:_POSIX_SAVED_IDS}},		/*   6, _SC_SAVED_IDS */
   {cons, {c:_POSIX_VERSION}},		/*   7, _SC_VERSION */
